module.exports = (project) => {
 project.compilers.register('library', () => ({
    configure() {
      return project
        .compiler('node')
        .configure(c => c
          .entry({
            index: project.join('src', 'index.js'),
            properties: 'properties.js',
            query: 'query.js',
            registry: 'registry.js',
            'webpack-compat': 'webpack-compat.js',
            router: 'router.js',
            entity: 'entity.js',
            string: 'string.js',
            'path-matcher': 'path-matcher.js',
            'path-to-regexp': 'path-to-regexp.js',
            'object-hash': 'object-hash.js',
            'emitter': 'emitter.js',
            'middleware': 'mware.js',
            testRule: 'path-matcher.js',
            inflect: 'inflect.js',
            emitter: 'emitter.js',
            mware: 'mware.js',
          })
          .output({
            path: project.join('lib'),
            libraryTarget: 'commonjs2',
            filename: '[name].js',
          })
          .modules(project.join('src'))
        ).config
    }
  }))

  project.compilers.register('bundle', () => ({
    configure(options = {}) {
      const minify = options.minify || project.argv.minify

      return project
        .compiler('node', { sourcemaps: false })
        .configure(c => c
          .entry({ bundle: project.join('src', 'index.js') })
          .plugin('webpack.ProvidePlugin', {
            lodash: 'expose?lodash!lodash/lodash.min.js'
          })
          .output({
            path: project.join('dist'),
            libraryTarget: 'umd',
            filename: `skypager-util${ minify ? '.min' : '' }.js`,
          })
          .when(minify, (cfg) => cfg
            .plugin('webpack.optimize.UglifyJsPlugin', {
              compress: { warnings: false },
            })
          )

        ).config
    }
  }))
}
