export class EventEmitter {
  events = {}

  on(eventName, fn) {
    if(!this.events[eventName]) {
      this.events[eventName] = [];
    }

    this.events[eventName].push(fn);
  }

  once(eventName, fn) {
    const self = this;

    this.on(eventName, function once() {
      fn.apply(null, arguments);
      self.off(eventName, once);
    });
  }

  removeAllListeners() {
    Object.keys(this.events).forEach(function(eventName) {
      delete(events[eventName])
    })
  }

  off (eventName, fn) {
    if(!fn) {
      delete this.events[eventName];
      return;
    }

    var index = this.events[eventName].indexOf(fn);
    if(index !== -1) {
      this.events[eventName].splice(index, 1);
      if(this.events[eventName].length === 0) {
        delete this.events[eventName];
      }
    }
  }

  removeListener(eventName, fn) {
     return this.off(eventName, fn)
  }

  emit(eventName, ...args) {
    if(!this.events[eventName]) return;

    this.events[eventName].forEach(function(fn) {
      fn.apply(null, args);
    });
  }

  static attachTo = attachTo
  static attachEmitter = attachTo
  static attach = attachTo
}

export default EventEmitter

function hide(target, property, value, configurable) {
  Object.defineProperty(target, property, { value, configurable: !!configurable, enumerable: false })
  return target
}

export function attachTo (host) {
  const emitter = new EventEmitter()

  hide(host, 'emitter', emitter)
  hide(host, 'on', emitter.on.bind(emitter))
  hide(host, 'addListener', emitter.on.bind(emitter))
  hide(host, 'once', emitter.once.bind(emitter))
  hide(host, 'off', emitter.off.bind(emitter))
  hide(host, 'removeAllListeners', emitter.removeAllListeners.bind(emitter))
  hide(host, 'emit', emitter.emit.bind(emitter))
  hide(host, 'trigger', emitter.emit.bind(emitter))

  return emitter
}

export const attachEmitter = attachTo

export const attach = attachTo


